#ifndef LSTORE_BASE_ALLOCATOR_H
#define LSTORE_BASE_ALLOCATOR_H

#include <vector>

#include "Base/Spinlock.h"
#include "Base/types.h"

namespace lstore {

class Allocator {
 public:
  /**
   * @param[in] block_size - size of the fixed blocks.
   * @param[in] blocks_count - number of blocks in the pool.
   * @param[in] infinity_pool - if true, then when one pool ends,
   *    the next one will be allocated from main memory,
   *    otherwise there will be only one pool.
   */
  Allocator(usize block_size, usize block_count, bool infinity_pool);

  ~Allocator();

  /**
   * Returns pointer to the block. Otherwise nullptr if unsuccessful.
   */
  void* allocate();

  /**
   * Returns a pointer to the memory pool.
   */
  void deallocate(void* block);

  usize getBlockSize() { return block_size_; }

 private:
  struct Block {
    Block* next;
  };

  usize block_size_;
  usize block_count_;
  bool is_infinity_pool_;
  Block* free_blocks_head_ = nullptr;
  std::vector<uint8*> pools_;
  uint8* current_pool_ = nullptr;
  uint32 pool_index_ = 0;
  Spinlock spin_;

  /**
   * Push a memory block onto head of free-list.
   */
  void push(void* ptr);

  /**
   * Pop a memory block from head of free-list.
   */
  void* pop();
};

} // namespace lstore

#endif // LSTORE_BASE_ALLOCATOR_H
