#include "Base/Spinlock.h"

namespace lstore {

Spinlock::Spinlock() {
  spin_.store(false, std::memory_order_release);
}

bool Spinlock::isLocked() const {
  return spin_.load(std::memory_order_relaxed);
}

bool Spinlock::tryLock() {
  bool cur = spin_.exchange(true, std::memory_order_acquire);
  return !cur;
}

void Spinlock::lock() {
  while (!tryLock()) {
    while (spin_.load(std::memory_order_acquire)) {
      // do noting
    }
  }
}

void Spinlock::unlock() {
  spin_.store(false, std::memory_order_release);
}

} // namespace lstore
