#ifndef LSTORE_CATALOG_CATALOG_H
#define LSTORE_CATALOG_CATALOG_H

#include <string>
#include <unordered_map>

#include "Base/types.h"
#include "Catalog/ColumnDesc.h"
#include "Catalog/File.h"
#include "Catalog/Table.h"

namespace lstore {

class Catalog {
 public:
  Catalog();
  ~Catalog();

  Catalog(const Catalog&) = delete;
  Catalog& operator=(const Catalog&) = delete;
  Catalog(Catalog&&) = delete;
  Catalog& operator=(Catalog&&) = delete;

  TablePtr& getTable(oid id);

  bool containsTable(oid id);

  FilePtr& getFile(oid id);

  TablePtr& createTable(const ColumnDescs& user_columns, const std::string& name);

 private:
  oid table_id_seq_ = 0;
  oid file_id_seq_ = 0;
  std::unordered_map<oid, TablePtr> tables_;
  std::unordered_map<oid, FilePtr> files_;

  void readFromFile();

  void writeToFile();
};

} // namespace lstore

#endif // LSTORE_CATALOG_CATALOG_H
