#ifndef LSTORE_CATALOG_COLUMN_H
#define LSTORE_CATALOG_COLUMN_H

#include <string>
#include <vector>

#include "Base/json_utils.h"
#include "Base/types.h"
#include "DataTypes/TypeId.h"

namespace lstore {

class ColumnDesc {
 public:
  ColumnDesc() = default;
  ColumnDesc(const std::string& name, TypeId type_id, bool have_index = false);

  ColumnDesc(const ColumnDesc&) = default;
  ColumnDesc& operator=(const ColumnDesc&) = default;
  ColumnDesc(ColumnDesc&&) = default;
  ColumnDesc& operator=(ColumnDesc&&) = default;

  const std::string& getName() const { return name_; }

  TypeId getType() const { return type_id_; }

  bool haveIndex() const { return have_index_; }

  friend void from_json(const json& j, ColumnDesc& c);
  friend void to_json(json& j, ColumnDesc& c);

 private:
  std::string name_;
  TypeId type_id_;
  bool have_index_;
};

void from_json(const json& j, ColumnDesc& c);
void to_json(json& j, ColumnDesc& c);

using ColumnDescs = std::vector<ColumnDesc>;

} // namespace lstore

#endif // LSTORE_CATALOG_COLUMN_H
