#ifndef LSTORE_CATALOG_FILE_H
#define LSTORE_CATALOG_FILE_H

#include <cstddef>
#include <memory>
#include <string>

#include "Base/json_utils.h"
#include "Base/types.h"
#include "Pages/PageId.h"

namespace lstore {

class File {
 public:
  File() = default;

  File(const std::string& name, oid id);

  File(const File&) = delete;
  File& operator=(const File&) = delete;
  File(File&&) = delete;
  File& operator=(File&&) = delete;

  const std::string& getName() const { return name_; }

  oid getId() const { return id_; }

  friend void from_json(const json& j, File& f);
  friend void to_json(json& j, File& f);

 private:
  std::string name_;
  oid id_;
};

void from_json(const json& j, File& f);
void to_json(json& j, File& f);

using FilePtr = std::shared_ptr<File>;

} // namespace lstore

#endif // LSTORE_CATALOG_FILE_H
