#ifndef LSTORE_CATALOG_TABLE_H
#define LSTORE_CATALOG_TABLE_H

#include <memory>
#include <string>
#include <vector>

#include "Base/json_utils.h"
#include "Base/types.h"
#include "Catalog/ColumnDesc.h"
#include "Indexes/Index.h"

namespace lstore {

enum MetaColumns : uint8 {
  kIndirection = 0,
  kSchemaEncoding = 1,
  kStartTime = 2,
  kPageTypeDependent = 3
};

constexpr uint32 kMetaColumnsCount = 4;

class Table {
 public:
  Table() = default;
  Table(const ColumnDescs& user_columns, const std::string& name, oid id,
        oid first_file_oid);

  void init();

  Table(const Table&) = delete;
  Table& operator=(const Table&) = delete;
  Table(Table&&) = delete;
  Table& operator=(Table&&) = delete;

  const ColumnDescs& getColumns() const { return columns_; }

  const std::vector<usize>& getUserColumnIndexes() const {
    return user_column_indexes_;
  }

  usize getUserColumnCount() const { return user_column_indexes_.size(); }

  const std::string& getName() const { return name_; }

  IndexPtr& getIndex() { return index_; }

  oid getColumnBaseFileOid(uint32 column_index) const;

  oid getId() const { return id_; }

  friend void from_json(const json& j, Table& t);
  friend void to_json(json& j, Table& t);

 private:
  ColumnDescs columns_;
  std::string name_;
  std::vector<usize> user_column_indexes_;
  oid id_;
  oid first_file_oid_;
  IndexPtr index_;

  void fillByUserColumns(const ColumnDescs& user_columns);

  void init_index();
};

void from_json(const json& j, Table& t);
void to_json(json& j, Table& t);

using TablePtr = std::shared_ptr<Table>;

} // namespace lstore

#endif // LSTORE_CATALOG_TABLE_H
