#ifndef LSTORE_CORE_RESULTSET_H
#define LSTORE_CORE_RESULTSET_H

#include <string>
#include <vector>

#include "Core/Value.h"

namespace lstore {

class ResultSet {
 public:
  ResultSet(const std::vector<Row> &result)
    : result_(result), is_error_ocurred_(false)
  {}

  ResultSet(const std::string &error_message)
    : error_message_(error_message), is_error_ocurred_(true)
  {}

  ResultSet(const ResultSet&) = default;
  ResultSet(ResultSet&&) = default;
  ResultSet& operator=(const ResultSet&) = default;
  ResultSet& operator=(ResultSet&&) = default;

  const std::vector<Row>& getResult() const { return result_; }

  bool errorOcurred() const { return is_error_ocurred_; }

  const std::string& getErrorMessage() const { return error_message_; }

 private:
  std::vector<Row> result_;
  std::string error_message_;
  bool is_error_ocurred_;
};

} // namespace lstore

#endif // LSTORE_CORE_RESULTSET_H
