#include "Core/Value.h"

namespace lstore {

Value& Value::operator=(const Value& v) {
  destroy();
  // This assigns string or vector without deallocation of existing buffer.
  assign(v);
  return *this;
}

Value& Value::operator=(Value&& v) {
  destroy();
  // This assigns string or vector without deallocation of existing buffer.
  assign(std::move(v));
  return *this;
}

Value& Value::operator=(std::string_view str) {
  destroy();
  assignString(str.data(), str.size());
  return *this;
}

Value& Value::operator=(std::string&& str){
  destroy();
  assignString(std::move(str));
  return *this;
}

void Value::assignString(const char* data, usize size) {
  std::string str{data, size};
  storage_ = std::move(str);
  type_ = TypeId::String;
}

void Value::assignString(std::string&& str) {
  storage_ = std::move(str);
  type_ = TypeId::String;
}

template<typename E>
static constexpr auto toIntegral(E e) -> typename std::underlying_type<E>::type  {
  return static_cast<typename std::underlying_type<E>::type>(e);
}

void Value::assign(const Value& x) {
  storage_ = x.storage_;
  type_ = x.type_;
}

void Value::assign(Value&& x) {
  storage_ = std::move(x.storage_);
  type_ = x.type_;
}

void Value::destroy() {
  if (type_ == TypeId::String) {
    std::string& v = get<std::string>();
    v.~basic_string();
  }
}

} // namespace lstore
