#ifndef LSTORE_DATATYPES_DATATYPENUMBER_H
#define LSTORE_DATATYPES_DATATYPENUMBER_H

#include <cstddef>
#include <type_traits>

#include "DataTypes/DataTypes.h"
#include "DataTypes/IDataType.h"
#include "DataTypes/NumberSerialization.h"
#include "DataTypes/TypeId.h"
#include "DataTypes/TypeName.h"

namespace lstore {

template <typename T>
class DataTypeNumber final : public IDataType {

  static_assert(std::is_arithmetic_v<T>, "Number must be arithmetic");

 public:
  static constexpr auto kName = TypeName<T>;
  static constexpr auto kTypeId = TypeToTypeId<T>;
  static NumberSerialization<T> kSerialization;

  DataTypeNumber() = default;

  std::string_view getName() const override { return kName; }

  TypeId getTypeId() const override { return kTypeId; }

  bool haveMaximumSizeOfValue() const override { return true; }

  usize getSizeOfValueInMemory() const override { return sizeof(T); }

  ISerialization& getSerialization() override { return kSerialization; }

};

template<typename T>
NumberSerialization<T> DataTypeNumber<T>::kSerialization;

extern template class DataTypeNumber<UInt8>;
extern template class DataTypeNumber<UInt16>;
extern template class DataTypeNumber<UInt32>;
extern template class DataTypeNumber<UInt64>;

extern template class DataTypeNumber<Int8>;
extern template class DataTypeNumber<Int16>;
extern template class DataTypeNumber<Int32>;
extern template class DataTypeNumber<Int64>;

extern template class DataTypeNumber<Float32>;
extern template class DataTypeNumber<Float64>;

using DataTypeUInt8 = DataTypeNumber<UInt8>;
using DataTypeUInt16 = DataTypeNumber<UInt16>;
using DataTypeUInt32 = DataTypeNumber<UInt32>;
using DataTypeUInt64 = DataTypeNumber<UInt64>;

using DataTypeInt8 = DataTypeNumber<Int8>;
using DataTypeInt16 = DataTypeNumber<Int16>;
using DataTypeInt32 = DataTypeNumber<Int32>;
using DataTypeInt64 = DataTypeNumber<Int64>;

using DataTypeFloat32 = DataTypeNumber<Float32>;
using DataTypeFloat64 = DataTypeNumber<Float64>;

} // namespace lstore

#endif // LSTORE_DATATYPES_DATATYPENUMBER_H
