#ifndef LSTORE_DATATYPES_IDATATYPE_H
#define LSTORE_DATATYPES_IDATATYPE_H

#include <cstddef>
#include <memory>
#include <string>

#include "Base/types.h"
#include "DataTypes/ISerialization.h"
#include "DataTypes/TypeId.h"

namespace lstore {

/**
 * Properties of data type.
 * Implementations of this interface represent a data type (example: UInt8)
 * DataType is totally immutable object.
 */
class IDataType {
 public:
  IDataType() = default;
  virtual ~IDataType() = default;

  IDataType(const IDataType&) = delete;
  IDataType& operator=(const IDataType&) = delete;
  IDataType(IDataType&&) = delete;
  IDataType& operator=(IDataType&&) = delete;

  /**
   * Name of data type (examples: UInt64, String).
   */
  virtual std::string_view getName() const = 0;

  /**
   * Data type id.
   */
  virtual TypeId getTypeId() const = 0;

  /**
   * Example: numbers.
   * Counterexamples: String.
   */
  virtual bool haveMaximumSizeOfValue() const { return false; }

  /**
   * Size in amount of bytes in memory. Returns 0 if not haveMaximumSizeOfValue.
   */
  virtual usize getSizeOfValueInMemory() const { return 0; }

  virtual ISerialization& getSerialization() = 0;

};

using DataTypePtr = std::shared_ptr<IDataType>;

} // namespace lstore

#endif // LSTORE_DATATYPES_IDATATYPE_H
