#ifndef LSTORE_DATATYPES_NUMBERSERIALIZATION_H
#define LSTORE_DATATYPES_NUMBERSERIALIZATION_H

#include <cassert>
#include <cstring>

#include "DataTypes/DataTypes.h"
#include "DataTypes/ISerialization.h"
#include "DataTypes/TypeId.h"

namespace lstore {

template <typename T>
class NumberSerialization final : public ISerialization {

  static_assert(std::is_arithmetic_v<T>, "Number must be arithmetic");

 public:
  NumberSerialization() = default;
  ~NumberSerialization() = default;

  void serialize(uint8* dest, Value val) override {
    assert((TypeToTypeId<T> == val.getType()) && "Value type doesn't match Serialization type");
    usize len = sizeof(T);
    T& data = val.get<T>();
    memcpy(dest, &data, len);
  }

  Value deserialize(uint8* src, usize len) override {
    assert(len == sizeof(T));
    T data;
    memcpy(&data, src, len);
    return data;
  }

};

} // namespace lstore

#endif // LSTORE_DATATYPES_NUMBERSERIALIZATION_H
