#ifndef LSTORE_DATATYPES_TYPEID_H
#define LSTORE_DATATYPES_TYPEID_H

#include "DataTypes/DataTypes.h"

namespace lstore {

enum class TypeId : uint8 {
  Nothing = 0,

  UInt8 = 1,
  UInt16 = 2,
  UInt32 = 3,
  UInt64 = 4,

  Int8 = 5,
  Int16 = 6,
  Int32 = 7,
  Int64 = 8,

  Float32 = 9,
  Float64 = 10,

  String = 11,
};

/**
 * Obtain TypeId value from real type if possible.
 * Returns TypeId::Nothing if type was not present in TypeId;
 * Returns TypeId element otherwise.
 * @example TypeToTypeId<UInt8> == TypeId::UInt8
 * @example TypeToTypeId<MySuperType> == TypeId::Nothing
 */
template <class T> inline constexpr TypeId TypeToTypeId = TypeId::Nothing;

#define TYPEID_MAP(_A) \
  template <> inline constexpr TypeId TypeToTypeId<_A> = TypeId::_A;

TYPEID_MAP(UInt8)
TYPEID_MAP(UInt16)
TYPEID_MAP(UInt32)
TYPEID_MAP(UInt64)

TYPEID_MAP(Int8)
TYPEID_MAP(Int16)
TYPEID_MAP(Int32)
TYPEID_MAP(Int64)

TYPEID_MAP(Float32)
TYPEID_MAP(Float64)

TYPEID_MAP(String)

#undef TYPEID_MAP

} // namespace lstore

#endif // LSTORE_DATATYPES_TYPEID_H
