#ifndef LSTORE_PAGES_BASERANGE_H
#define LSTORE_PAGES_BASERANGE_H

#include <vector>

#include "Base/types.h"
#include "Catalog/Table.h"
#include "Core/Value.h"
#include "Pages/Location.h"
#include "Pages/Page.h"
#include "Pages/PageRanges.h"
#include "Pages/TailRange.h"

namespace lstore {

// forward decl
class MergeManager;

class BaseRange {
public:
  BaseRange(Location location, TablePtr& table);

  Value read(rid id, usize column_number);

  void write(rid id, usize column_number);

  /**
   * Returns tuple of meta column values.
   */
  Row readMeta(rid id);

  void update(xid xid, rid id, usize column_number,
              Value new_value, MergeManager& merge_manager);

  void del(xid xid, rid id, MergeManager& merge_manager);

  uint32 getTailRangeNumber() { return tail_range_number_; }

  rid getTps() { return tps_; }

  void setTailRangeNumber(uint32 tail_range_number) { tail_range_number_ = tail_range_number; }

  PageRange& getPages() { return pages_; }

private:
  TablePtr& table_;
  rid first_;
  Location location_;
  uint32 tail_range_number_;
  rid tps_;
  PageRange pages_;
};

} // namespace lstore

#endif // LSTORE_PAGES_BASERANGE_H
