#ifndef LSTORE_PAGES_MERGEMANAGER_H
#define LSTORE_PAGES_MERGEMANAGER_H

#include <atomic>
#include <condition_variable>
#include <mutex>
#include <queue>
#include <thread>

#include "Base/types.h"
#include "Pages/BaseRange.h"

namespace lstore {

// forward decl
class PageManager;

struct MergeRequest {
  oid table_id;
  uint32 base_range_number;

  MergeRequest() = default;

  MergeRequest(oid table_id, uint32 base_range_number)
    : table_id(table_id), base_range_number(base_range_number)
  {}
};

class MergeManager {
 public:
  MergeManager(PageManager& page_manager);

  ~MergeManager();

  void pushRequest(MergeRequest request);

  void stopMerging();

 private:
  std::queue<MergeRequest> requests_;
  std::atomic_bool running_;
  std::mutex mx_;
  std::thread worker_;
  std::condition_variable consumerCond_;
  std::condition_variable producerCond_;
  PageManager& page_manager_;
};

} // namespace lstore

#endif // LSTORE_PAGES_MERGEMANAGER_H
