#ifndef LSTORE_PAGES_PAGEMANAGER_H
#define LSTORE_PAGES_PAGEMANAGER_H

#include <unordered_map>
#include <vector>

#include "Base/types.h"
#include "Pages/BaseRange.h"
#include "Pages/InsertRange.h"

namespace lstore {

class PageManager {
 public:
  PageManager() = default;

  bool isLoadedTable(oid table_id) {
    return ranges_.contains(table_id);
  }

  void loadTable(TablePtr& table);

  InsertRange& getInsertRange(oid table_id) {
    return ranges_[table_id]->insert_range;
  }

 private:
  struct TableRanges {
    rid rid_seq = 0;
    InsertRange insert_range;
    std::vector<BaseRange> base_ranges;

    TableRanges(TablePtr& table);

    TableRanges(const TableRanges&) = default;
    TableRanges& operator=(const TableRanges&) = default;
    TableRanges(TableRanges&&) = default;
    TableRanges& operator=(TableRanges&&) = default;
  };

  std::unordered_map<oid, std::shared_ptr<TableRanges>> ranges_;
};

} // namespace lstore

#endif // LSTORE_PAGES_PAGEMANAGER_H
