#include <iostream>
#include <string>

#include "Core/Storage.h"
#include "Core/Value.h"
#include "Queries/InsertQuery.h"
#include "DataTypes/TypeRegistry.h"

using namespace lstore;
using namespace std::string_literals;

int main() {
  TypeRegistry::init();
  Storage storage;
  DbContext& context = storage.getContext();
  Catalog& catalog = context.getCatalog();

  oid table_id = 0;
  if (!catalog.containsTable(table_id)) {
    ColumnDescs columns = { ColumnDesc{"Id"s, TypeId::UInt64, true},
                          ColumnDesc{"A"s, TypeId::UInt64, false},
                          ColumnDesc{"B"s, TypeId::UInt64, false},
                          ColumnDesc{"C"s, TypeId::UInt64, false},
                          ColumnDesc{"D"s, TypeId::UInt64, false},
                          ColumnDesc{"E"s, TypeId::Float64, false}, };
    catalog.createTable(columns, "Table"s);
  }
  TablePtr& table = catalog.getTable(table_id);

  for (UInt64 i = 0; i < 1000ull; i += 1) {
    InsertQuery query{table->getId(), storage, {i, 1ull, 1ull, 1ull, 1ull, 1.0}};

    ResultSet result = query.execute();

    if (result.errorOcurred()) {
      std::cout << "Error: " << result.getErrorMessage() << std::endl;
    } else if (!result.getResult().empty()) {
      std::cout << "Error: Not empty insert result" << std::endl;
    } else {
      std::cout << "Success" << std::endl;
    }
  }
}
